{----------------------------------------------------------------------------}
{                                                                            }
{ Program:    NODE-MGR.PAS                                                   }
{ Purpose:    Allow "non-network" door programs to be used on a network      }
{ Language:   Borland's Turbo Pascal Version 3.01                            }
{                                                                            }
{ Author:     Carl M. Evans                                                  }
{             Vervan's War Board BBS                                         }
{             P.O. Box 1176, Cucamonga, CA 91730  Voice: 714-989-7824        }
{             2400/1200/300 Baud (8-N-1 only)      Data: 714-989-7596        }
{                                                                            }
{ Notice:     This program is placed in the public domain and may be         }
{             freely used by anyone for any purpose.  When using this        }
{             code, please give credit to the original author and to         }
{             anyone who has subsequently improved or modified it.           }
{                                                                            }
{ Narrative:  This Turbo Pascal program is intended for use on PCBOARD 14.0  }
{             multi-node BBS's. NODE-MGR allows "non-network" doors to be    }
{             safely run on a network by preventing more than one node from  }
{             accessing the same door at the same time. It is suggested that }
{             you "install" NODE-MGR by copying it to a directory which is on}
{             the DOS's PATH. Then for each door you want "controlled", you  }
{             should edit the batch control files on each node to add a call }
{             to NODE-MGR from the same directory as your PCBOARD.SYS file.  }
{             NODE-MGR will look for a special "node lock" file in the target}
{             directory (where the door is). If the door is NOT in being used}
{             by another node, then NODE-MGR will create a new "node lock"   }
{             file and exit to DOS with an errorlevel of zero. If NODE-MGR   }
{             finds an existing "node lock" file, then it will read the com  }
{             port settings from PCBOARD.SYS, issue a message to the caller, }
{             and exit to DOS with an errorlevel of one. The batch file can  }
{             test the DOS errorlevel to determine whether or not to allow   }
{             the caller to enter the door program.                          }
{                                                                            }
{             The default parameters used by NODE-MGR are:                   }
{             C:\PCB\PCBOARD.SYS [location of the BBS data file]             }
{             NODE-MGR.KEY       ["node lock" file in the door directory]    }
{             You can override these defaults with command line parameters.  }
{             For example:                                                   }
{             NODE-MGR D:\PCB\PCBOARD.SYS E:\DOORS\TW500\NODE-MGR.KEY        }
{                                                                            }
{                                                                            }
{ History:    Version 1.0, 04/16/88 - Original by Carl M. Evans              }
{                                     for PCBOARD 12.1                       }
{                                                                            }
{                     2.0, 09/06/88 - Updated for PCBOARD 14.0               }
{                                                                            }
{                     2.1, 09/07/88 - Corrected bug in Baud Rate conversion  }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{                                                                            }
{----------------------------------------------------------------------------}
{.PA}
Program NODE_MGR;
{$U-}
{$C-}
{$P256}
{$G256}
{$I COMM-IO.INC}
{============================================================================}
{ Declarations                                                               }
{============================================================================}

Const
  PCB14FileDef    = 'C:\PCB\PCBOARD.SYS';
  NodeLockFileDef = 'NODE-MGR.KEY';
  VersionNumber   = '2.1';


Type
  Ascii2          = Array[1.. 2] of char;
  Ascii4          = Array[1.. 4] of char;
  Ascii5          = Array[1.. 5] of char;
  Ascii12         = Array[1..12] of char;
  Ascii15         = Array[1..15] of char;
  Ascii25         = Array[1..25] of char;
  String5         = String[5];
  String6         = String[6];
  String80        = String[80];
  FileNameType    = String[80];

  PCB14FileType   = Record             { 128 Bytes for PCBOARD version 14.0  }
         Display  : Ascii2;            { Sysop's Display ON/OFF              }
         Printer  : Ascii2;            { Sysop's Printer ON/OFF              }
         PageBell : Ascii2;            { Sysop's Page Bell ON/OFF            }
         CallAlarm: Ascii2;            { Caller Alarm ON/OFF                 }
         SysopNext: Char;              { Sysop "Next On" Flag                }
         ModemType: Ascii2;            { Error Correcting Modem YES/NO       }
         Graphics : Char;              { Graphics Mode of Caller             }
         NodeChat : Char;              { Available for Node Chat YES/NO      }
         BaudOpen : Ascii5;            { Baud Rate to Open Modem at          }
         BaudRate : Ascii5;            { "CONNECT" Baud Rate of Caller       }
         RecordNum: Integer;           { User's Database Record Number (MKI$)}
         FirstName: Ascii15;           { First Name of Caller                }
         Password : Ascii12;           { Password of Caller                  }
         TimeLogOn: Integer;           { Time User Logged On   (MKI$ minutes)}
         TimeUsed : Integer;           { Time Used Today       (MKI$ minutes)}
         LogOnStr : Ascii5;            { Time User Logged On       (HH:MM:SS)}
         TimeLimit: Integer;           { Daily Time Limit      (MKI$ minutes)}
         ByteLmtDL: Integer;           { Daily Download Limit  (MKI$ K-bytes)}
         ConfExit : Char;              { Conference (Area) User Exited From  }
         ConfFlags: Ascii5;            { Areas User Has "Joined" Today       }
         ConfScan : Ascii5;            { Areas User Has Scanned for New Mail }
         ConfTime : Integer;           { Area "Bonus" Time     (MKI$ minutes)}
         TimeCred : Integer;           { Upload Time Credit    (MKI$ minutes)}
         Language : Ascii4;            { Language Version Being Used         }
         Name     : Ascii25;           { Full Name of Caller   (First & Last)}
         TimeLeft : Integer;           { Session Time Remaining(MKI$ minutes)}
         NodeNum  : Char;              { BBS Node Number That Caller is Using}
         EventTime: Ascii5;            { Scheduled EVENT Time         (HH:MM)}
         EventFlag: Ascii2;            { Is EVENT Time Active?               }
         EventMove: Ascii2;            { Delay EVENT Until User Logs Off?    }
         MsgRecall: Ascii4;            { "Memorized" Message Number    (MKS$)}
         ComPortID: Char;              { Com Port Number (0, 1, 2)           }
         Reserved : Ascii2;            { Reserved for Future Use             }
       End;

Var

{ Variables used to define disk files                                        }
  PCB14File             : File of PCB14FileType;
  PCB14FileName         : FileNameType;
  PCB14Data             : PCB14FileType;
  NodeLockFile          : Text;
  NodeLockFileName      : FileNameType;

{ Miscellaneous Variables                                                    }
  Local                 : Boolean;
  ErrorCode             : Integer;
  DoorKeyStatus         : String6;
  TempChar              : Char;

{============================================================================}
{ Function to trim leading blanks from a string                              }
{============================================================================}
Function TrimL(InpStr: String80): String80;

Var
  Len,
  I              : Integer;

Begin { TrimL }
  Len := Length(InpStr);
  I := 1;
  While (I <= Len) and (InpStr[I] = ' ') do
    I := I + 1;
    TrimL := Copy(InpStr,I,Len-I+1)
End;  { TrimL }

{============================================================================}
{ Function to trim trailing blanks from a string                             }
{============================================================================}
Function TrimR(InpStr: String80): String80;

Var
  I             : Integer;

Begin { TrimR }
  I := Length(InpStr);
  While (I >= 1) and (InpStr[I] = ' ') do
    I := I - 1;
    TrimR := Copy(InpStr,1,I)
End;  { TrimR }

{============================================================================}
{ Function to convert a string to upper case characters                      }
{============================================================================}
Function UpperCase(InpStr: String80): String80;

Var
  I : Integer;

Begin { UpperCase }
   For I := 1 to Length(InpStr) do
       UpperCase[I] := UpCase(InpStr[I]);
   UpperCase[0] := InpStr[0]
End;  { UpperCase }

{============================================================================}
{ Procedure to make an annoying BEEP sound                                   }
{============================================================================}
Procedure Beep;

Begin { Beep }
  Sound(450);
  Delay(500);
  NoSound;
End;  { Beep }

{============================================================================}
{ Procedure to display command line syntax for sysop when error occurs       }
{============================================================================}
Procedure ErrorMsg;

Begin { ErrorMsg }
  Beep;
  Writeln(' ');
  Writeln('FIRST PARAMETER (optional) -');
  Writeln(' The default filespec for the 14.0 file is:  C:\PCB\PCBOARD.SYS');
  Writeln(' If you wish to override the default path for PCBOARD.SYS,');
  Writeln(' then specify a new path as the first parameter.');
  Writeln(' NOTE: Include the file name with the path.');
  Writeln(' ');
  Writeln('SECOND PARAMETER (optional) -');
  Writeln(' The default output filespec is:  NODE-MGR.KEY');
  Writeln(' If you wish to override the default path for NODE-MGR.KEY,');
  Writeln(' then specify a new path as the second parameter.');
  Writeln(' NOTE: Include the file name with the path.');
  Writeln(' ');
  Writeln('For Example:');
  Writeln('NODE-MGR D:\PCB\PCBOARD.SYS E:\DOORS\TW500\NODE-MGR.KEY');
  HALT(1);
End;  { ErrorMsg }

{============================================================================}
{ Procedure to set the input and output file names                           }
{============================================================================}
Procedure GetFileNames;

Begin { GetFileNames }
  If ParamCount > 0 then begin
    PCB14FileName := ParamStr(1);
    PCB14FileName := TrimL(PCB14FileName);
    PCB14FileName := TrimR(PCB14FileName);
    PCB14FileName := UpperCase(PCB14FileName);
  End else begin
    PCB14FileName := PCB14FileDef;
  End; {endif}
  If ParamCount > 1 then begin
    NodeLockFileName := ParamStr(1);
    NodeLockFileName := TrimL(NodeLockFileName);
    NodeLockFileName := TrimR(NodeLockFileName);
    NodeLockFileName := UpperCase(NodeLockFileName);
  End else begin
    NodeLockFileName := NodeLockFileDef;
  End; {endif}
End;  { GetFileNames }

{============================================================================}
{ Procedure to determine if the door is locked or not                        }
{============================================================================}
Procedure RequestDoorAccess;

Begin { RequestDoorAccess }
  Assign(NodeLockFile,NodeLockFileName);
  {$I-} Reset(NodeLockFile); {$I+}
  ErrorCode := IOResult;
  If ErrorCode = 0 then begin
    DoorKeyStatus := 'CLOSED';
  End else begin
    DoorKeyStatus := 'OPEN';
  End; {endif}
  Close(NodeLockFile);
End; { RequestDoorAccess }

{============================================================================}
{ Procedure to read User data from PCBOARD.SYS file (version 14.0)           }
{============================================================================}
Procedure ReadPCB14File;

Begin { ReadPCB14File }
  Writeln(' ');
  Writeln('CURRENT INPUT FILESPEC: ' + PCB14FileName);
  Assign(PCB14File,PCB14FileName);
  {$I-} Reset(PCB14File); {$I+}
  ErrorCode := IOresult;
  If ErrorCode <> 0 then begin
    Close(PCB14File);
    Writeln(' ');
    Writeln('ERROR: Could not find PCBOARD.SYS file');
    ErrorMsg;
  End; {endif}
  Read(PCB14File,PCB14Data);
  Close(PCB14File);
End;  { ReadPCB14File }

{============================================================================}
{ Procedure to initialize com port variables                                 }
{============================================================================}
Procedure SetComPortData;

Var
  I              : Integer;
  TempString,
  BaudRate       : String5;

Begin { SetComPortData }
  For I := 1 to 5 do TempString[I] := PCB14Data.BaudRate[I];
  BaudRate := TempString;
  Val(BaudRate,Baud,ErrorCode);

  If PCB14Data.Graphics = '7' then begin
    DataBits := 7;
    Parity   := Even;
  End else begin
    DataBits := 8;
    Parity   := None;
  End; {endif}

  StopBits := 1;

  Local := False;
  Case PCB14Data.ComPortID of
    '2'                      : ComPort := 2;
    '1'                      : ComPort := 1;
  Else                         Begin
                                 ComPort := 0;
                                 Local   := True;
                               End;
  End; {endcase}
End;  { SetComPortData }

{============================================================================}
{ Procedure to send a string to the com port                                 }
{============================================================================}
Procedure SendString(Msg : String80);

Var
  I   : Integer;

Begin { SendString }
  Writeln(Msg);
  If (NOT Local) and (NOT Carrier) then begin
    Writeln('');
    Writeln('Lost carrier...');
    CloseComPort;
    HALT(1);
  End; {endif}
  If NOT Local then begin
    If Length(Msg) > 0 then begin
      For I := 1 to Length(Msg) do begin
        WriteCom(Msg[I]);
      End; {endfor}
    End; {endif}
    WriteCom(^M);
    WriteCom(^J);
  End; {endif}
End; { SendString }

{============================================================================}
{ Procedure to tell Caller that access is denied                             }
{============================================================================}
Procedure DenyAccess;

Begin { DenyAccess }
  If NOT Local then OpenComPort;
  SendString(' ');
  SendString('Sorry, but the DOOR you requested is in use by another Node.');
  SendString(' ');
  SendString('Please try again later...');
  SendString(' ');
  SendString('Returning to PCBOARD...');
  If NOT Local then CloseComPort;
  HALT(1);
End;  { DenyAccess }

{============================================================================}
{ Procedure to grant access to the door and set the node lock                }
{============================================================================}
Procedure GrantAccess;

Begin { GrantAccess }
  Assign(NodeLockFile,NodeLockFileName);
  {$I-} Rewrite(NodeLockFile); {$I+}
  ErrorCode := IOresult;
  If ErrorCode <> 0 then begin
    Close(NodeLockFile);
    Writeln(' ');
    Writeln('ERROR: Output Filespec is not legal.');
    ErrorMsg;
  End; {endif}
  Writeln(NodeLockFile,'Delete this file before returning Caller to BBS');
  Close(NodeLockFile);
  HALT(0);
End;  { GrantAccess }

{============================================================================}
{ Main Program                                                               }
{============================================================================}

Begin { Main Program }
  Writeln('');
  Writeln('NODE-MGR version ' + VersionNumber);
  Writeln('');
  Writeln('[ A PCBoard 14.0 utility from Vervan' + Chr(39) +
    's War Board (714) 989-7596 ]');
  Writeln('');
  GetFileNames;
  RequestDoorAccess;
  If DoorKeyStatus = 'CLOSED' then begin
    ReadPCB14File;
    SetComPortData;
    DenyAccess;
  End else GrantAccess;
End. { Main Program }

{===[ END OF PROGRAM ]=======================================================}
